local addon, shared = ...

local RELEASE=		99
local TESTING=		20
local WDOGYIELD=	15
local FUNCTIONRUN=	10
local STATUSUPDT=	8
local INFORMATION=	5
local DETAIL=		1

---------------------
-- Variables
---------------------

_Rifter = {}
local RFT = _Rifter

local UIFrames = { abw = 0, abilities = {}, categories = {}, POPUPANCHOR = "CENTER" }

local UINeeded = true
local SVLoaded = false
local RSreqd = false
local coroDead = false

local RFTYPE_SPELL = 1
local RFTYPE_ITEM  = 2

local RFMODE_POPUP = 1
local RFMODE_CLICK = 2

local RFMODE_IMG = {
	"textures/RFMODE_POPUP.png", "textures/RFMODE_CLICK.png"
	}

local RFSHOW_ALL	= 1
local RFSHOW_AVL	= 2

local rfmenuVis = false
local itemidcnt = 0

local frW = 0
local popupFade = false
local MACROUPDATE = {}

local default_settings = {
	mmx = -1, mmy = -1, itemid = {}, locked = false, mode = RFMODE_POPUP, fs=14, scale = 1, folded = {}, show = RFSHOW_AVL, fade=1
	}

local __abilities = {}
local __items = {}

---------------------
-- Functions
---------------------

local debuglevel = RELEASE

local function Debug(s,l)
	if (l or INFORMATION) >= debuglevel then
		print(s)
	end
end

local _watchdogyield = {}
local yieldcnt = 0
local function WATCHDOGWAIT(x)
	local rmx = Inspect.System.Watchdog()
	while rmx < 0.03 do
		yieldcnt = yieldcnt+1
		Debug("#"..yieldcnt..":rmx: "..rmx.." ["..x.."]",WDOGYIELD)
		_watchdogyield[yieldcnt] = { tx = os.date("%Y%m%d-%H:%M:%S"), m = x, tr = rmx }
		coroutine.yield()
		rmx = Inspect.System.Watchdog()
	end
end

local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

function RFT.Dump()
	local d = Inspect.Item.Detail("si01.001")
	print(string.format("{ name=\"%s\", id=\"%s\", icon=\"%s\", type=RFTYPE_ITEM, itemname=\"%s\"},",
		d.name, d.type, d.icon, d.name))
end

function RFT.ShowPopup()
	if (Inspect.System.Secure() == true) or (Rifter_Settings.locked == false) then
		return
	end

	if MINIMAPDOCKER then
		MINIMAPDOCKER.AUTOHIDE_DOCKED = false
	end
	RFT.AttachPopup()
	rfmenuVis = true
	--RFT.ResizeWindow()
	UIFrames.frame:SetVisible(true)
end

function RFT.ResizeWindow()
	local cur_spell = UIFrames.headerlft
	local cur_item = UIFrames.headerrgt
	local lastframe
	local frmh = 0
	local ncat_l = 0
	local rift = { [RFTYPE_SPELL] = 0, [RFTYPE_ITEM] = 0 }
	local ncat_r = 0
	local rift_f = 0
	local showsec = false

	for k,v in ipairs(shared.riftabilities) do
		showsec = false
		lastframe = UIFrames.categories[k].frame
		for k1, v1 in ipairs(v.data) do
			local show = false
			if v1.rftype == RFTYPE_SPELL then
				if __abilities[v1.id].active then
					show = true
				end
			else
				if __items[v1.id].active then
					show = true
				end
			end

			if show == true or Rifter_Settings.show == RFSHOW_ALL then
				show = true
				showsec = true
				if Rifter_Settings.folded[v.type] then
					show = false
				else
					UIFrames.abilities[v1.id].frame:SetPoint("TOPLEFT", lastframe, "BOTTOMLEFT", 0,0)
					lastframe = UIFrames.abilities[v1.id].frame
					rift[v1.rftype] = rift[v1.rftype]+1
				end
			end

			UIFrames.abilities[v1.id].frame:SetVisible(show)
		end

		if showsec then
			if v.rftype == RFTYPE_SPELL then
				ncat_l = ncat_l+1
				UIFrames.categories[k].frame:SetPoint("TOPLEFT", cur_spell, "BOTTOMLEFT", 0,0)
				cur_spell = lastframe
			else
				ncat_r = ncat_r+1
				UIFrames.categories[k].frame:SetPoint("TOPLEFT", cur_item, "BOTTOMLEFT", 0,0)
				cur_item = lastframe
			end
		end
		UIFrames.categories[k].frame:SetVisible(showsec)
	end

	frmh = math.max(((ncat_l*UIFrames.htxt16)+(rift[RFTYPE_SPELL]*UIFrames.hmacro)),((ncat_r*UIFrames.htxt16)+(rift[RFTYPE_ITEM]*UIFrames.hmacro)))
	UIFrames.frame:SetHeight((2*UIFrames.htxt16)+frmh)
end

function RFT.ReScale()
	local frmw = 0

	for k,v in pairs(UIFrames.categories) do
		v.frame:ClearWidth()
		frmw = math.max(frmw, v.frame:GetWidth())
	end

	UIFrames.hmacro = 0

	for k, v in pairs(UIFrames.abilities) do
		v.label:SetFontSize(Rifter_Settings.fs)
		if UIFrames.hmacro == 0 then
			UIFrames.hmacro = v.label:GetHeight()
		end
		v.frame:SetHeight(UIFrames.hmacro)
		v.icon:SetWidth(UIFrames.hmacro)
		v.icon:SetHeight(UIFrames.hmacro)
		frmw = math.max(frmw, (UIFrames.hmacro+v.label:GetWidth()))
	end

	UIFrames.header:SetWidth(2+frmw+frmw)
	UIFrames.header:SetHeight(UIFrames.htxt16)
	UIFrames.headerlft:SetWidth(frmw)
	UIFrames.headerlft:SetHeight(UIFrames.htxt16)
	UIFrames.headerrgt:SetWidth(frmw)
	UIFrames.headerrgt:SetHeight(UIFrames.htxt16)
	UIFrames.frame:SetWidth(2+frmw+frmw)

	for k,v in pairs(UIFrames.abilities) do
		v.frame:SetWidth(frmw)
	end

	for k,v in pairs(UIFrames.categories) do
		v.frame:SetWidth(frmw)
	end

	RFT.ResizeWindow()
end

function RFT.AttachPopup()
	local h_loc
	local v_loc
	local anchor

	if UIFrames.mm_button:GetLeft() >= (UIParent:GetWidth()/2) then
		h_loc = "RIGHT"
	else
		h_loc = "LEFT"
	end

	if UIFrames.mm_button:GetTop() >= (UIParent:GetHeight()/2) then
		v_loc = "BOTTOM"
	else
		v_loc = "TOP"
	end

	anchor = string.format("%s%s", v_loc, h_loc)

	if Inspect.System.Secure() == false then
		if anchor ~= UIFrames.POPUPANCHOR then
			UIFrames.frame:ClearPoint(UIFrames.POPUPANCHOR)
			UIFrames.POPUPANCHOR = anchor
			UIFrames.frame:SetPoint(anchor, UIFrames.mm_button, "CENTER", 0,0)
		end
	end
end

function RFT.ClosePopup()
	if Inspect.System.Secure() == false then
		rfmenuVis = false
		UIFrames.frame:SetVisible(false)
	end
end

function RFT.CreateUI()
	RFT.context = UI.CreateContext(addon.identifier)
	RFT.context:SetSecureMode("restricted")

	UIFrames.mm_button = UI.CreateFrame("Texture", "UIFrames.mm_button", RFT.context)
	UIFrames.mm_button:SetVisible(false)
	UIFrames.mm_button:SetHeight(36)
	UIFrames.mm_button:SetWidth(36)
	UIFrames.mm_button:SetLayer(2)
	UIFrames.mm_button:SetTexture(addon.identifier, "textures/mm_button.png")

	if MINIMAPDOCKER then
		MINIMAPDOCKER.Register(addon.identifier, UIFrames.mm_button, RFT.ClosePopup)
	else
		UIFrames.mm_button:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
			if Inspect.System.Secure() == false then
				if Rifter_Settings.locked == false then
					local mouseData = Inspect.Mouse()
					self.sx = mouseData.x - UIFrames.mm_button:GetLeft()
					self.sy = mouseData.y - UIFrames.mm_button:GetTop()
					self.MouseDown = true
				end
			else
				self.MouseDown = false
			end
		end, "Event.UI.Input.Mouse.Left.Down")

		UIFrames.mm_button:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
			if Inspect.System.Secure() == false then
				if self.MouseDown and Rifter_Settings.locked == false then
					local nx, ny
					local mouseData = Inspect.Mouse()
					nx = mouseData.x - self.sx
					ny = mouseData.y - self.sy
					UIFrames.mm_button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
				end
			else
				self.MouseDown = false
			end
		end, "Event.UI.Input.Mouse.Cursor.Move")

		UIFrames.mm_button:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
			self.MouseDown = false
			Rifter_Settings.mmx = UIFrames.mm_button:GetLeft()
			Rifter_Settings.mmy = UIFrames.mm_button:GetTop()
		end, "Event.UI.Input.Mouse.Left.Up")

		UIFrames.mm_button:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
			if Inspect.System.Secure() == false then
				Rifter_Settings.locked = not Rifter_Settings.locked
				if Rifter_Settings.locked then
					UIFrames.mm_button:SetTexture(addon.identifier, "textures/mm_button.png")
				else
					UIFrames.mm_button:SetTexture(addon.identifier, "textures/mm_button_bw.png")
					rfmenuVis = false
					UIFrames.frame:SetVisible(false)
					popupFade = false
				end
			else
				print("System in secure state. Too dangerous to unlock right now. Try again later.")
			end
		end, "Event.UI.Input.Mouse.Right.Click")

	end

	UIFrames.mm_button:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, function(self, h)
		if Inspect.System.Secure() == false then
			if Rifter_Settings.locked or MINIMAPDOCKER then
				if Rifter_Settings.fs < 18 then
					Rifter_Settings.fs = Rifter_Settings.fs+1
					RFT.ReScale()
				end
			end
		else
			print("System in secure state. Cannot resize elements.")
		end
	end, "Event.UI.Input.Mouse.Wheel.Forward")

	UIFrames.mm_button:EventAttach(Event.UI.Input.Mouse.Wheel.Back, function(self, h)
		if Inspect.System.Secure() == false then
			if Rifter_Settings.locked or MINIMAPDOCKER then
				if Rifter_Settings.fs > 7 then
					Rifter_Settings.fs = Rifter_Settings.fs-1
					RFT.ReScale()
				end
			end
		else
			print("System in secure state. Cannot resize elements.")
		end
	end, "Event.UI.Input.Mouse.Wheel.Back")

	UIFrames.mm_button:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self, h)
		if Inspect.System.Secure() == false then
			if  (Rifter_Settings.locked or MINIMAPDOCKER) and Rifter_Settings.mode == RFMODE_POPUP then
				RFT.ShowPopup()
			end
		else
			self.MouseDown = false
		end
	end, "Event.UI.Input.Mouse.Cursor.In")

	UIFrames.mm_button:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if Inspect.System.Secure() == false then
			if (Rifter_Settings.locked or MINIMAPDOCKER) and Rifter_Settings.mode == RFMODE_CLICK then
				RFT.ShowPopup()
			end
		else
			self.MouseDown = false
		end
	end, "Event.UI.Input.Mouse.Left.Click")

	WATCHDOGWAIT("CreateUI:1")

	UIFrames.frame = UI.CreateFrame("Frame", "UIFrames.frame", RFT.context)
	UIFrames.frame:SetSecureMode("restricted")
	UIFrames.frame:SetVisible(false)
	UIFrames.frame:SetLayer(0)
	UIFrames.frame:SetBackgroundColor(0,0,0,0.5)

	UIFrames.header= UI.CreateFrame("Frame", "UIFrames.header", UIFrames.frame)
	UIFrames.header:SetPoint("TOPLEFT", UIFrames.frame, "TOPLEFT", 0,0)
	UIFrames.header:SetLayer(8)
	UIFrames.header:SetBackgroundColor(0,0,1)

	UIFrames.headertxt= UI.CreateFrame("Text", "UIFrames.headertxt", UIFrames.frame)
	UIFrames.headertxt:SetFontSize(16)
	UIFrames.headertxt:SetText("Open which rift...")
	UIFrames.htxt16=UIFrames.headertxt:GetHeight()
	UIFrames.hmacro=0
	UIFrames.headertxt:SetPoint("TOPCENTER", UIFrames.header, "TOPCENTER", 0,0)
	UIFrames.headertxt:SetFontColor(1,1,1)
	UIFrames.headertxt:SetLayer(10)

	UIFrames.header_icon = UI.CreateFrame("Texture", "UIFrames.header_icon", UIFrames.frame)
	UIFrames.header_icon:SetHeight(24)
	UIFrames.header_icon:SetPoint("TOPRIGHT", UIFrames.frame, "TOPRIGHT")
	UIFrames.header_icon:SetLayer(UIFrames.frame:GetLayer()+20)

	UIFrames.header_icon:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if Inspect.System.Secure() == false then
			if Rifter_Settings.mode == RFMODE_POPUP then
				Rifter_Settings.mode = RFMODE_CLICK
			else
				Rifter_Settings.mode = RFMODE_POPUP
			end
			UIFrames.header_icon:SetTexture(addon.identifier, RFMODE_IMG[Rifter_Settings.mode])
		end
	end, "Event.UI.Input.Mouse.Left.Click")

	UIFrames.headerlft = UI.CreateFrame("Text", "UIFrames.headerlft", UIFrames.frame)
	UIFrames.headerlft:SetFontSize(16)
	UIFrames.headerlft:SetText("ABILITIES")
	UIFrames.headerlft:SetPoint("TOPLEFT", UIFrames.header, "BOTTOMLEFT", 0,0)
	UIFrames.headerlft:SetFontColor(1,1,1)
	UIFrames.headerlft:SetBackgroundColor(0,0,1)
	UIFrames.headerlft:SetLayer(10)

	UIFrames.headerrgt = UI.CreateFrame("Text", "UIFrames.headerrgt", UIFrames.frame)
	UIFrames.headerrgt:SetFontSize(16)
	UIFrames.headerrgt:SetText("ITEM LURES")
	UIFrames.headerrgt:SetPoint("TOPLEFT", UIFrames.headerlft, "TOPRIGHT", 2,0)
	UIFrames.headerrgt:SetFontColor(1,1,1)
	UIFrames.headerrgt:SetBackgroundColor(0,0,1)
	UIFrames.headerrgt:SetLayer(10)

--	local prvframe = UIFrames.header
	local frmw = 0
	local hdrh = UIFrames.headertxt:GetHeight()
	local sech = 0

	for k,v in ipairs(shared.riftabilities) do
		WATCHDOGWAIT("CreateUI:2:"..k)
		local h = UI.CreateFrame("Text", "h", UIFrames.frame)
		h:SetFontSize(16)
		h:SetBackgroundColor(0,0,0.8)
		--if k == 3 then
		--	h:SetPoint("TOPLEFT", UIFrames.categories[1].frame, "TOPRIGHT", 2,0)
		--else
		--	h:SetPoint("TOPLEFT", prvframe, "BOTTOMLEFT", 0,0)
		--end
		h:SetText(v.type)
		sech = h:GetHeight()
		h:SetLayer(10)
		frmw = math.max(frmw, h:GetWidth())
		prvframe = h
		local i = UI.CreateFrame("Texture", "i", h)
		i:SetWidth(sech)
		i:SetHeight(sech)
		i:SetTexture(addon.identifier, "textures/rollup.png")
		i:SetPoint("TOPRIGHT", h, "TOPRIGHT", 0,0)
		i:SetLayer(12)
		i:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
			if Rifter_Settings.folded[v.type] then
				Rifter_Settings.folded[v.type] = nil
				i:SetTexture(addon.identifier, "textures/rollup.png")
				RFT.ResizeWindow()
			else
				Rifter_Settings.folded[v.type] = true
				i:SetTexture(addon.identifier, "textures/rolldown.png")
				RFT.ResizeWindow()
			end
		end, "Event.UI.Input.Mouse.Left.Click")
		UIFrames.categories[k] = {frame = h, icon = i}
		for k1,v1 in ipairs(v.data) do
			WATCHDOGWAIT("CreateUI:3:"..k1)
			local f = UI.CreateFrame("Frame", "f", UIFrames.frame)
			f:SetSecureMode("restricted")
			f:SetHeight(24)
			--f:SetPoint("TOPLEFT", prvframe, "BOTTOMLEFT", 0,0)
			f:SetLayer(10)
			--prvframe = f
			f:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self, h)
				f:SetBackgroundColor(0,1,0,0.5)
			end, "Event.UI.Input.Mouse.Cursor.In")
			f:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self, h)
				f:SetBackgroundColor(0,0,0,0)
			end, "Event.UI.Input.Mouse.Cursor.Out")
			local i = UI.CreateFrame("Texture", "i", f)
			i:SetWidth(24)
			i:SetHeight(24)
			i:SetTexture("Rift", v1.icon)
			i:SetPoint("TOPLEFT", f, "TOPLEFT", 0,0)
			i:SetLayer(10)
			local l = UI.CreateFrame("Text", "l", f)
			l:SetPoint("CENTERLEFT", i, "CENTERRIGHT",0,0)
			--l:SetFontSize(12)
			l:SetLayer(10)
			l:SetText("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")
			frmw = math.max(frmw, 24+l:GetWidth())
			if v1.rftype == RFTYPE_SPELL then
				if __abilities[v1.id].active then
					l:SetText(__abilities[v1.id].name)
					table.insert(MACROUPDATE, {f=f, m=string.format("cast %s", __abilities[v1.id].name)})
					--f:EventMacroSet(Event.UI.Input.Mouse.Left.Click, string.format("cast %s", __abilities[v1.id].name))
				else
					l:SetText(__abilities[v1.id].en_name)
				end
			else
				if __items[v1.id].active then
					local mn = ""
					for token in string.gmatch(__items[v1.id].name, "[^%s]+") do
						mn=token
					end
					l:SetText(mn)
					table.insert(MACROUPDATE, {f=f, m=string.format("use %s", __items[v1.id].name)})
					--f:EventMacroSet(Event.UI.Input.Mouse.Left.Click, string.format("use %s", __items[v1.id].name))
				else
					l:SetText(__items[v1.id].en_name)
				end
			end
			UIFrames.abilities[v1.id] = {frame=f, icon=i, label=l}
		end
	end

	--RFT.ResizeWindow()

--[[	UIFrames.header:SetHeight(hdrh)
	UIFrames.header:SetWidth(2+frmw+frmw)
	UIFrames.frame:SetWidth(2+frmw+frmw)
	UIFrames.frame:SetHeight((2*sech)+(19*24)+hdrh)

	for k,v in pairs(UIFrames.abilities) do
		v.frame:SetWidth(frmw)
	end

	for k,v in pairs(UIFrames.categories) do
		v.frame:SetWidth(frmw)
	end
]]--
	WATCHDOGWAIT("CreateUI:4")
end

function RFT.SVLoaded()
	if Rifter_Settings == nil then
		Rifter_Settings = {}
	end
	MergeTable(Rifter_Settings, default_settings)

	if MINIMAPDOCKER then
		Rifter_Settings.locked = true
	else
		if Rifter_Settings.mmx == -1 then
			UIFrames.mm_button:SetPoint("TOPCENTER", UI.Native.MapMini, "BOTTOMCENTER", 0,0)
			Rifter_Settings.mmx = UIFrames.mm_button:GetLeft()
			Rifter_Settings.mmy = UIFrames.mm_button:GetTop()
			UIFrames.mm_button:ClearPoint("TOPCENTER")
		end
		UIFrames.mm_button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Rifter_Settings.mmx, Rifter_Settings.mmy)
	end

	if Rifter_Settings.locked then
		UIFrames.mm_button:SetTexture(addon.identifier, "textures/mm_button.png")
	else
		UIFrames.mm_button:SetTexture(addon.identifier, "textures/mm_button_bw.png")
	end

	UIFrames.mm_button:SetVisible(true)
	RFT.AttachPopup()

	for k,v in pairs(UIFrames.categories) do
		local n = shared.riftabilities[k].type
		if Rifter_Settings.folded[n] then
			v.icon:SetTexture(addon.identifier, "textures/rolldown.png")
		end
	end
	UIFrames.header_icon:SetTexture(addon.identifier, RFMODE_IMG[Rifter_Settings.mode])
	RSreqd = true
end

local combatwarning = false

function RFT.checkMouse()
	local md = Inspect.Mouse()
	local l = math.min(UIFrames.frame:GetLeft(), UIFrames.mm_button:GetLeft())
	local r = math.max(UIFrames.frame:GetRight(), UIFrames.mm_button:GetRight())
	local t = math.min(UIFrames.frame:GetTop(), UIFrames.mm_button:GetTop())
	local b = math.max(UIFrames.frame:GetBottom(), UIFrames.mm_button:GetBottom())

	if md.x < l or md.x > r or md.y < t or md.y > b then
		if popupFade == false then
			popupFade = Inspect.Time.Real()
		end
	else
		popupFade = false
		UIFrames.frame:SetAlpha(1)
	end
end

function RFT.UpdateHandler()
	local frcnt = 0
	while true do
		if rfmenuVis == true then
			RFT.checkMouse()
		end
		if UINeeded then
			if Inspect.System.Secure() == false then
				if combatwarning then
					print("Out of combat, continuing with setup")
					combatwarning = false
				end
				RFT.CreateUI()
				UINeeded = false
			else
				if combatwarning == false then
					print("Setup delayed due to being in combat!")
					combatwarning = true
				end
			end
		elseif SVLoaded then
			RFT.SVLoaded()
			SVLoaded = false
		elseif #MACROUPDATE > 0 then
			if Inspect.System.Secure() == false then
				for x=#MACROUPDATE, 1, -1 do
					MACROUPDATE[x].f:EventMacroSet(Event.UI.Input.Mouse.Left.Click, MACROUPDATE[x].m)
					table.remove(MACROUPDATE, x)
				end
			end
		elseif RSreqd then
			if Inspect.System.Secure() == false then
				RFT.ReScale()
				RSreqd = false
			end
		elseif popupFade then
			local dx = Inspect.Time.Real()-popupFade
			local alpha = (Rifter_Settings.fade-dx)/Rifter_Settings.fade
			if Inspect.System.Secure() == false then
				if dx > Rifter_Settings.fade then
					popupFade = false
					rfmenuVis = false
					UIFrames.frame:SetVisible(false)
					UIFrames.frame:SetAlpha(1)
				else
					UIFrames.frame:SetAlpha(alpha)
				end
			end

		end
		coroutine.yield()
	end
end

RFT.coUpdateHandler = coroutine.create(RFT.UpdateHandler)

function RFT.Event_System_Update_Begin(h)
	if coroDead then return end
	local ok, errormsg = coroutine.resume(RFT.coUpdateHandler)
	if not ok then
		coroDead = true
		error("Error in coroutine: "..errormsg)
	end
end

function RFT.Event_System_Secure_Enter(h)
	if rfmenuVis == true then
		UIFrames.frame:SetVisible(false)
		rfmenuVis = false
	end
end

function RFT.Command_Slash_Register(h,args)
	local r = {}
	local numargs = 0
	for token in string.gmatch(args, "[^%s]+") do
		table.insert(r, token)
	end
	numargs=#r
	if r[1] == "mode" then
		if Inspect.System.Secure() == false then
			if Rifter_Settings.mode == RFMODE_POPUP then
				Rifter_Settings.mode = RFMODE_CLICK
				print("Mode set to LEFT-CLICK")
			else
				Rifter_Settings.mode = RFMODE_POPUP
				print("Mode set to POPUP")
			end
			UIFrames.header_icon:SetTexture(addon.identifier, RFMODE_IMG[Rifter_Settings.mode])
		else
			print("Cant change settings whilst in combat. Try again later.")
		end
	elseif r[1] == "show" then
		if Rifter_Settings.show == RFSHOW_AVL then
			Rifter_Settings.show = RFSHOW_ALL
			print("Showing ALL abilities. WARNING: This is a long list!")
		else
			Rifter_Settings.show = RFSHOW_AVL
			print("Showing AVAILABLE abilities")
		end
	elseif r[1] == "fade" then
		if tonumber(r[2]) == nil then
			if r[2] == nil then
				print("Fade time currently: "..Rifter_Settings.fade.."s")
			else
				print(string.format("'%s' is not a valid number", r[2]))
			end
		else
			Rifter_Settings.fade = tonumber(r[2])
			print("Setting fade time to: "..Rifter_Settings.fade.."s")
		end
	elseif r[1] == "reset" then
		if MINIMAPDOCKER then
			print("Not resetting button position. Location manged by Docker.")
		else
			if Inspect.System.Secure() == false then
				Rifter_Settings.mmx = 100
				Rifter_Settings.mmy = 100
				print("Relocating control button to 100,100")
				UIFrames.mm_button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Rifter_Settings.mmx, Rifter_Settings.mmy)
			else
				print("Unable to relocate button when in combat. Try again later.")
			end
		end
	else
		print("/rifter mode - Toggles between popup and right-click modes to display selection")
		print("/rifter show - Toggles between showing ALL or AVAILABLE rift summons")
		print("/rifter fade N - Sets the fadeout time when your mouse leaves the popup menu before the menu is removed.")
		print("/rifter reset - Relocates control button to center of screen.")
	end
end

function RFT.Event_Unit_Availability_Full(h,t)
	for k,v in pairs(t) do
		if v == "player" then
			Command.Event.Detach(Event.Unit.Availability.Full, nil, nil, nil, addon.identifier)
			LibVersionCheck.register(addon.toc.Identifier, addon.toc.Version)
		end
	end
end

function RFT.Event_Ability_New_Remove(h,t)
	local rs_needed = false
	for k,v in pairs(t) do
		if __abilities[k] ~= nil then
			__abilities[k].active = false
			rs_needed = true
		end
	end
	if rs_needed then
		RSreqd = true
	end
end

function RFT.Event_Ability_New_Add(h,t)
	local rs_needed = false
	for k,v in pairs(t) do
		if __abilities[k] ~= nil then
			rs_needed = true
			__abilities[k].active = true
			if __abilities[k].name == nil then
				local d = Inspect.Ability.New.Detail(k)
				__abilities[k].name = d.name
				if UIFrames.abilities[k] then
					UIFrames.abilities[k].label:SetText(d.name)
					table.insert(MACROUPDATE, {f=UIFrames.abilities[k].frame, m=string.format("cast %s", d.name)})
					--UIFrames.abilities[k].frame:EventMacroSet(Event.UI.Input.Mouse.Left.Click, string.format("cast %s", d.name))
				end
			end
		end
	end
	if rs_needed then
		RSreqd = true
	end
end

function RFT.Event_Addon_SavedVariables_Load_End(h, a)
	if a == addon.identifier then
		SVLoaded = true
	end
end

function RFT.cb_libinvITEM(t)
	local rs_needed = false
	for k,v in pairs(t) do
		local found = false
		if v["PLAYER"] then
			for slt,cnt in pairs(v["PLAYER"]) do
				if Library.LibInventory.REFSLOTS["inventory"][slt] then
					found = true
				end
			end
		end
		if found then
			rs_needed = true
			__items[k].active = true
			if __items[k].name == nil then
				local dn = Library.LibInventory.ITEMTYPE_NAME[k]
				__items[k].name = dn
				if UIFrames.abilities[k] then
					UIFrames.abilities[k].label:SetText(dn)
					table.insert(MACROUPDATE, {f=UIFrames.abilities[k].frame, m=string.format("use %s", dn)})
				end
			end
		else
			__items[k].active = false
		end
	end
	if rs_needed then
		RSreqd = true
	end
end

---------------------
-- Register Events
---------------------

for k,v in pairs(shared.riftabilities) do
	for k1, v1 in pairs(v.data) do
		if v1.rftype == RFTYPE_SPELL then
			__abilities[v1.id] = {active=false, name=nil, en_name = "*"..v1.name}
		else
			__items[v1.id] = {active=false, name=nil, en_name = "*"..v1.name }
			Library.LibInventory.RegisterItem(v1.id, RFT.cb_libinvITEM)
		end
	end
end

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, RFT.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Event.System.Update.Begin, RFT.Event_System_Update_Begin, "Event.System.Update.Begin")
Command.Event.Attach(Event.Unit.Availability.Full, RFT.Event_Unit_Availability_Full, "Event.Unit.Availability.Full")
Command.Event.Attach(Event.System.Secure.Enter, RFT.Event_System_Secure_Enter, "Event.System.Secure.Enter")
Command.Event.Attach(Command.Slash.Register("rifter"), RFT.Command_Slash_Register, "Command.Slash.Register")
Command.Event.Attach(Event.Ability.New.Add, RFT.Event_Ability_New_Add, "Event.Ability.New.Add")
Command.Event.Attach(Event.Ability.New.Remove, RFT.Event_Ability_New_Remove, "Event.Ability.New.Remove")

print(string.format("v%s loaded.", addon.toc.Version))
